/***************************************************************************
          eboxy-remote.c - command-line client for eboxy rcon plugin
                             -------------------
    begin                : Sun Apr 11 2004
    copyright            : (C) 2004 by Paul Eggleton
    email                : bluelightning\bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#if defined (READLINE_LIBRARY)
#  include "readline.h"
#  include "history.h"
#else
#  include <readline/readline.h>
#  include <readline/history.h>
#endif

#define DEFAULT_PORT 5450

int main (int argc, char *argv []) {
  int sockfd;
  struct sockaddr_in their_addr;
  struct hostent *he;
  int numbytes;
  char *inputline;
  int port;

  if(argc > 1 && argv[1][1] == '-') {
    printf("eboxy-remote 0.1\n");
    printf("usage:\n\n");
    printf("  %s [hostname[:port] [action]]\n\n", argv[0]);
    printf("where hostname is the host name or IP address of the remote machine (default\n");
    printf("127.0.0.1), port is the destination port (default %d) and action is a single\n", DEFAULT_PORT);
    printf("action to send. If action is not specified then eboxy-remote will start\n");
    printf("interactively - type exit or press Ctrl+D to quit interactive mode.\n");
    return 0;
  }
  
  if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
    perror("socket");
    return 1;
  }

  port = DEFAULT_PORT;
  if(argc > 1) {
    char *hostname;
    char *sepchr;

    hostname = strdup(argv[1]);
    sepchr = strchr(hostname, ':');
    if(sepchr) {
      port = atoi(sepchr+1);
      *sepchr = '\0';
    }

    /* Resolve the specified hostname */
    he = gethostbyname(hostname);
    if(he == NULL) {
      fprintf(stderr, "gethostbyname: %s: %s\n", hostname, hstrerror(h_errno));
      free(hostname);
      return 1;
    }

    free(hostname);
  }
  else
    he = NULL;

  their_addr.sin_family = AF_INET;
  their_addr.sin_port = htons(port);
  if(he)
    their_addr.sin_addr = *((struct in_addr *)he->h_addr);
  else
    their_addr.sin_addr.s_addr = inet_addr("127.0.0.1");

  memset(&(their_addr.sin_zero), '\0', 8); /* zero the rest of the struct */

  while(1) {
    if(argc > 2) {
      /* Just one action to send */
      inputline = argv[2];
    }
    else {
      /* Get input from stdin */
      inputline = readline(NULL);

      /* Test for EOF */
      if (inputline == 0) {
        close(sockfd);
        return 1;
      }
      else if(strcmp(inputline, "exit") == 0) {
        free(inputline);
        close(sockfd);
        return 0;
      }
      /* Add line to readline history */
      add_history(inputline);
    }

    if ((numbytes=sendto(sockfd, inputline, strlen(inputline), 0,
        (struct sockaddr *)&their_addr, sizeof(struct sockaddr))) == -1) {
      perror("sendto");
      close(sockfd);
      return 1;
    }

    if(argc > 2) {
      close(sockfd);
      return 0;
    }

    free(inputline);
  }
}
