/***************************************************************************
                  rcon.c - eboxy remote control server plugin
                             -------------------
    begin                : Sun Apr 11 2004
    copyright            : (C) 2004 by Paul Eggleton
    email                : bluelightning\bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>
#include <errno.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
    
#include "eboxyplugin.h"
#include "pluginconstants.h"

#define DEFAULT_PORT 5450

int port;
char *address;
pthread_t listen_thread;
int sockfd;
struct sockaddr_in inaddr;
int listening;

/* Function prototypes */
char *rcon_getport(const char *sender);
int rcon_setport(const char *sender, const char *value);
char *rcon_getaddress(const char *sender);
int rcon_setaddress(const char *sender, const char *value);
char *rcon_listen(const char *sender, int numargs, const char *args[]);

int do_listen(void);
void *listen_threadfn(void *arg);

char *inttostr(int n);
int strtobool(const char *str);

/* Implementation */

int ebplugin_init(void) {
  int rc = 0;

  /* Set version info for the plugin */
  setPluginInfo("RemoteControlServer", "0.1");

  /* Register an object to be used in scripts */
  rc = registerObject("rcon");
  if(rc)
    return rc;
  rc = registerPropertyDL("rcon", "address", "rcon_getaddress", "rcon_setaddress");
  if(rc)
    return rc;
  rc = registerPropertyDL("rcon", "port", "rcon_getport", "rcon_setport");
  if(rc)
    return rc;
  rc = registerMethodDL("rcon", "listen", 0, "rcon_listen");
  if(rc)
    return rc;

  port = DEFAULT_PORT;
  address = strdup("127.0.0.1");
  listening = 0;

  /* Success */
  return 0;
}

int ebplugin_message(int msgcode, void *msgdata) {
  if(msgcode == PLMSG_AFTERPAGECHANGE) {
    if(!listening)
      do_listen();
  }

  return 0;     /* return 0 here for future compatibility */
}

void ebplugin_deinit(void) {
  free(address);
}

char *rcon_getport(const char *sender) {
  return inttostr(port);
}

int rcon_setport(const char *sender, const char *value) {
  int portnum;
  portnum = atoi(value);
  if(portnum > 0 && portnum < 65536) {
    port = portnum;
    return 0;
  }
  else
    return -1;
}

char *rcon_getaddress(const char *sender) {
  return strdup(address);
}

int rcon_setaddress(const char *sender, const char *value) {
  free(address);
  address = strdup(value);
  return 0;
}

char *rcon_listen(const char *sender, int numargs, const char *args[]) {
  if(!listening)
    do_listen();
  return NULL;
}

int do_listen(void) {
  sockfd = socket(AF_INET, SOCK_DGRAM, 0);
  inaddr.sin_family = AF_INET;         // host byte order
  inaddr.sin_port = htons(port);     // short, network byte order
  inaddr.sin_addr.s_addr = inet_addr(address);
  memset(&(inaddr.sin_zero), '\0', 8); // zero the rest of the struct

  // don't forget your error checking for bind():
  if(bind(sockfd, (struct sockaddr *)&inaddr, sizeof(struct sockaddr))  == -1) {
    perror("bind");
    return 1;
  }
  
  if(pthread_create(&listen_thread, NULL, listen_threadfn, NULL)) {
    fprintf(stderr, "rcon: failed to create listener thread\n");
    return 2;
  }

  listening = 1;

  return 0;
}

#define MAXBUFLEN 1024

void *listen_threadfn(void *arg) {
  struct sockaddr_in remaddr;
  int addr_len, numbytes;
  char buf[MAXBUFLEN];
       
  addr_len = sizeof(struct sockaddr);

  while(1) {
    if ((numbytes=recvfrom(sockfd,buf, MAXBUFLEN-1, 0,
                       (struct sockaddr *)&remaddr, &addr_len)) == -1) {
      perror("recvfrom");
      break;
    }

    buf[numbytes] = '\0';
    performAction(buf);

    // FIXME: no way to break out of here
  }

  close(sockfd);
  return NULL;
}


/* Convert an integer to a string */
char *inttostr(int n) {
  char *buf;
  buf = malloc(16);
  snprintf(buf, 16, "%d", n);
  return buf;
}
