/***************************************************************************
                      testbench.c - eboxy dynamic widget test plugin
                             -------------------
    begin                : Mon Jun 12 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "eboxyplugin.h"
#include "pluginconstants.h"

#define PLUGIN_OBJECT_NAME "testbench"
#define TESTBENCH_WIDGET_NAME "tbwidget"
#define TESTBENCH_WIDGET_CLONE_NAME "tbwidgetclone"
#define TESTBENCH_PAGE_NAME "tbtestpage"
#define TESTBENCH_PAGE_WIDGET_NAME "tbtestpagewidget"

/* Globals */
char *proplist;
char *propeditbox;
const char *oldpagename;

/* Function prototypes */
char *tbp_create(const char *sender, int numargs, const char *args[]);
char *tbp_destroy(const char *sender, int numargs, const char *args[]);
char *tbp_clone(const char *sender, int numargs, const char *args[]);
char *tbp_addtopage(const char *sender, int numargs, const char *args[]);
char *tbp_removefrompage(const char *sender, int numargs, const char *args[]);
char *tbp_initpropcontrols(const char *sender, int numargs, const char *args[]);
char *tbp_dopagetest(const char *sender, int numargs, const char *args[]);
char *tbp_donepagetest(const char *sender, int numargs, const char *args[]);
int tbp_proplistselchange(const char *sender);
int tbp_propsetbuttonclick(const char *sender);
int tbp_testpagebuttonclick(const char *sender);

void updatelists(void);
char *inttostr(int n);

/* Implementation */

int ebplugin_init(void) {
  int rc = 0;

  /* Set version info for the plugin */
  setPluginInfo("WidgetTestBench", "0.1");

  /* Register an object to be used in scripts */
  rc = registerObject(PLUGIN_OBJECT_NAME);
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "create", 1, "tbp_create");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "addtopage", 0, "tbp_addtopage");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "removefrompage", 0, "tbp_removefrompage");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "destroy", 0, "tbp_destroy");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "clone", 0, "tbp_clone");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "initpropcontrols", 3, "tbp_initpropcontrols");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "dopagetest", 0, "tbp_dopagetest");
  if(rc)
    return rc;
  rc = registerMethodDL(PLUGIN_OBJECT_NAME, "donepagetest", 0, "tbp_donepagetest");
  if(rc)
    return rc;

  proplist = NULL;
  propeditbox = NULL;

  printf("testbench initialised\n");

  /* Success */
  return 0;
}

int ebplugin_message(int msgcode, void *msgdata) {
  return 0;     /* return 0 here for future compatibility */
}

void ebplugin_deinit(void) {
  if(proplist)
    free(proplist);
  if(propeditbox)
    free(propeditbox);
}

char *tbp_create(const char *sender, int numargs, const char *args[]) {
  char *tmplt;

  tmplt = (char *)malloc(32);
  strcpy(tmplt, "t_");
  strcat(tmplt, args[0]);
  
  createWidget(TESTBENCH_WIDGET_NAME, args[0], tmplt);
  free(tmplt);

  setPropertyAsString(TESTBENCH_WIDGET_NAME, "x", "150");
  setPropertyAsString(TESTBENCH_WIDGET_NAME, "y", "150");

  updatelists();

  return NULL;
}

char *tbp_addtopage(const char *sender, int numargs, const char *args[]) {
  const char *cargs[1];
  
  cargs[0] = TESTBENCH_WIDGET_NAME;
  callMethodNoReturn("currentpage", "addwidget", 1, cargs);

  return NULL;
}

char *tbp_removefrompage(const char *sender, int numargs, const char *args[]) {
  const char *cargs[1];
  
  cargs[0] = TESTBENCH_WIDGET_NAME;
  callMethodNoReturn("currentpage", "removewidget", 1, cargs);
  cargs[0] = TESTBENCH_WIDGET_CLONE_NAME;
  callMethodNoReturn("currentpage", "removewidget", 1, cargs);

  return NULL;
}

char *tbp_destroy(const char *sender, int numargs, const char *args[]) {
  deleteWidget(TESTBENCH_WIDGET_NAME);
  deleteWidget(TESTBENCH_WIDGET_CLONE_NAME);

  updatelists();

  return NULL;
}

char *tbp_clone(const char *sender, int numargs, const char *args[]) {
  const char *cargs[1];
  
  cloneWidget(TESTBENCH_WIDGET_NAME, TESTBENCH_WIDGET_CLONE_NAME);

  setPropertyAsString(TESTBENCH_WIDGET_CLONE_NAME, "x", "170");
  setPropertyAsString(TESTBENCH_WIDGET_CLONE_NAME, "y", "170");

  cargs[0] = TESTBENCH_WIDGET_CLONE_NAME;
  callMethodNoReturn("currentpage", "addwidget", 1, cargs);

  return NULL;
}

char *tbp_initpropcontrols(const char *sender, int numargs, const char *args[]) {
  /* Initialise the widgets used for property viewing/editing */
  proplist = strdup(args[0]);
  propeditbox = strdup(args[1]);
  registerEventHandlerDL(proplist, "OnSelectionChange", "tbp_proplistselchange");
  registerEventHandlerDL(args[2], "OnClick", "tbp_propsetbuttonclick");

  return NULL;
}

char *tbp_dopagetest(const char *sender, int numargs, const char *args[]) {
  const char *cargs[1];

  /* Create a page */
  createPage(TESTBENCH_PAGE_NAME);

  /* Create a button */
  createWidget(TESTBENCH_PAGE_WIDGET_NAME, "button", "t_button");
  setPropertyAsString(TESTBENCH_PAGE_WIDGET_NAME, "x", "200");
  setPropertyAsString(TESTBENCH_PAGE_WIDGET_NAME, "y", "200");
  setPropertyAsString(TESTBENCH_PAGE_WIDGET_NAME, "caption", "<< Back");
  registerEventHandlerDL(TESTBENCH_PAGE_WIDGET_NAME, "OnClick", "tbp_testpagebuttonclick");

  /* Add button to page */
  cargs[0] = TESTBENCH_PAGE_WIDGET_NAME;
  callMethodNoReturn(TESTBENCH_PAGE_NAME, "addwidget", 1, cargs);

  oldpagename = getPropertyAsString("currentpage", "name");

  changePage(TESTBENCH_PAGE_NAME);

  return NULL;
}

char *tbp_donepagetest(const char *sender, int numargs, const char *args[]) {
  deletePage(TESTBENCH_PAGE_NAME);
  deleteWidget(TESTBENCH_PAGE_WIDGET_NAME);
  return NULL;
}

int tbp_proplistselchange(const char *sender) {
  char *propname;
  char *value;

  if(propeditbox) {
    if(findObject(TESTBENCH_WIDGET_NAME, NULL)) {
      propname = getPropertyAsString(proplist, "selectedtext");
      value = getPropertyAsString(TESTBENCH_WIDGET_NAME, propname);
      if(propname && value)
        setPropertyAsString(propeditbox, "text", value);
      if(propname)
        free(propname);
      if(value)
        free(value);
    }    
  }

  return 0;
}

int tbp_propsetbuttonclick(const char *sender) {
  char *propname;
  char *value;

  if(findObject(TESTBENCH_WIDGET_NAME, NULL)) {
    propname = getPropertyAsString(proplist, "selectedtext");
    value = getPropertyAsString(propeditbox, "text");
    if(propname && value)
      setPropertyAsString(TESTBENCH_WIDGET_NAME, propname, value);
    if(propname)
      free(propname);
    if(value)
      free(value);
  }

  return 0;
}

int tbp_testpagebuttonclick(const char *sender) {
  changePage(oldpagename);
  return 0;
}

void updatelists(void) {
  int count,i;
  char *propname;
  char *indexstr;
  const char *cargs[1];
  
  if(proplist) {
    callMethodNoReturn(proplist, "clear", 0, NULL);
    if(findObject(TESTBENCH_WIDGET_NAME, NULL)) {
      count = atoi(getPropertyAsString(TESTBENCH_WIDGET_NAME, "propertycount"));
      for(i=0;i<count;i++) {
        /* FIXME this is very sloppy */
        indexstr = inttostr(i);
        cargs[0] = indexstr;
        propname = callMethod(TESTBENCH_WIDGET_NAME, "getproperty", 1, cargs);
        free(indexstr);
        if(propname) {
          cargs[0] = propname;
          callMethodNoReturn(proplist, "additem", 1, cargs);
          free(propname);
        }
      }
    }
  }
}

/* Convert an integer to a string */
char *inttostr(int n) {
  char *buf;
  buf = malloc(16);
  snprintf(buf, 16, "%d", n);
  return buf;
}
