/***************************************************************************
                  xmmsctrl.c - eboxy XMMS control plugin
                             -------------------
    begin                : Sun Oct 13 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "eboxyplugin.h"
#include "pluginconstants.h"
#include <xmmsctrl.h>

/* Globals */
int session = 0;

/* Function prototypes */
char *xmmsp_getsession(const char *sender);
int xmmsp_setsession(const char *sender, const char *value);
char *xmmsp_getrunning(const char *sender);
char *xmmsp_getplaying(const char *sender);
char *xmmsp_getshowmainwindow(const char *sender);
int xmmsp_setshowmainwindow(const char *sender, const char *value);
char *xmmsp_getplaylistcount(const char *sender);
char *xmmsp_getplaylistpos(const char *sender);
int xmmsp_setplaylistpos(const char *sender, const char *value);
char *xmmsp_play(const char *sender, int numargs, const char *args[]);
char *xmmsp_stop(const char *sender, int numargs, const char *args[]);
char *xmmsp_pause(const char *sender, int numargs, const char *args[]);
char *xmmsp_exit(const char *sender, int numargs, const char *args[]);
char *xmmsp_prevtrack(const char *sender, int numargs, const char *args[]);
char *xmmsp_nexttrack(const char *sender, int numargs, const char *args[]);

char *inttostr(int n);
const char *booltostr(int n);
int strtobool(const char *str);

/* Plugin methods */
struct {
  char *methodname;
  int argc;
  char *methodfunc;
} plugin_methods[] = {
  {"play",       0, "xmmsp_play"},
  {"stop",       0, "xmmsp_stop"},
  {"pause",      0, "xmmsp_pause"},
  {"exit",       0, "xmmsp_exit"},
  {"prevtrack",  0, "xmmsp_prevtrack"},
  {"nexttrack",  0, "xmmsp_nexttrack"},
  {NULL, 0, NULL}
};

/* Plugin properties */
struct {
  char *propertyname;
  char *getterfunc;
  char *setterfunc;
} plugin_properties[] = {
  {"session",        "xmmsp_getsession",        "xmmsp_setsession"},
  {"running",        "xmmsp_getrunning",        NULL},
  {"playing",        "xmmsp_getplaying",        NULL},
  {"showmainwindow", "xmmsp_getshowmainwindow", "xmmsp_setshowmainwindow"},
  {"playlistcount",  "xmmsp_getplaylistcount",  NULL},
  {"playlistpos",    "xmmsp_getplaylistpos",    "xmmsp_setplaylistpos"},
  {NULL, 0, NULL}
};

/* Implementation */

int ebplugin_init(void) {
  int rc, i;

  /* Set version info for the plugin */
  setPluginInfo("XMMS Control", "0.1");

  /* Register an object to be used in scripts */
  rc = registerObject("xmmsctrl");
  if(rc)
    return rc;

  /* Register properties */
  for(i=0;;i++) {
    if(plugin_properties[i].propertyname == NULL)
      break;
    rc = registerPropertyDL("xmmsctrl", plugin_properties[i].propertyname, plugin_properties[i].getterfunc, plugin_properties[i].setterfunc);
    if(rc)
      return rc;
  }
  /* Register methods */
  for(i=0;;i++) {
    if(plugin_methods[i].methodname == NULL)
      break;
    rc = registerMethodDL("xmmsctrl", plugin_methods[i].methodname, plugin_methods[i].argc, plugin_methods[i].methodfunc);
    if(rc)
      return rc;
  }

  /* Success */
  return 0;
}

int ebplugin_message(int msgcode, void *msgdata) {
  /* Nothing to do */
  return 0;     /* return 0 here for future compatibility */
}

void ebplugin_deinit(void) {
  /* Nothing to do */
}

char *xmmsp_getsession(const char *sender) {
  return inttostr(session);
}

int xmmsp_setsession(const char *sender, const char *value) {
  session = atoi(value);
  return 0;
}

char *xmmsp_getrunning(const char *sender) {
  return strdup(booltostr(xmms_remote_is_running(session)));
}

char *xmmsp_getplaying(const char *sender) {
  return strdup(booltostr(xmms_remote_is_playing(session)));
}

char *xmmsp_getshowmainwindow(const char *sender) {
  return strdup(booltostr(xmms_remote_is_main_win(session)));
}

int xmmsp_setshowmainwindow(const char *sender, const char *value) {
  xmms_remote_main_win_toggle(session, strtobool(value));
  return 0;
}

char *xmmsp_getplaylistcount(const char *sender) {
  return inttostr(xmms_remote_get_playlist_length(session));
}

char *xmmsp_getplaylistpos(const char *sender) {
  return inttostr(xmms_remote_get_playlist_pos(session));
}

int xmmsp_setplaylistpos(const char *sender, const char *value) {
  xmms_remote_set_playlist_pos(session, atoi(value));
  return 0;
}

char *xmmsp_play(const char *sender, int numargs, const char *args[]) {
	xmms_remote_play(session);
  return NULL;
}

char *xmmsp_stop(const char *sender, int numargs, const char *args[]) {
  xmms_remote_stop(session);
  return NULL;
}

char *xmmsp_pause(const char *sender, int numargs, const char *args[]) {
	xmms_remote_pause(session);
  return NULL;
}

char *xmmsp_exit(const char *sender, int numargs, const char *args[]) {
	xmms_remote_quit(session);
  return NULL;
}

char *xmmsp_prevtrack(const char *sender, int numargs, const char *args[]) {
	xmms_remote_playlist_prev(session);
  return NULL;
}

char *xmmsp_nexttrack(const char *sender, int numargs, const char *args[]) {
	xmms_remote_playlist_next(session);
  return NULL;
}


/* Convert an integer to a string */
char *inttostr(int n) {
  char *buf;
  buf = malloc(16);
  snprintf(buf, 16, "%d", n);
  return buf;
}

/* Convert a boolean value to a string */
const char *booltostr(int n) {
  if(n)
    return "true";
  else
    return "false";
}

/* Convert a string to a boolean value */
int strtobool(const char *str) {
  if(strcasecmp(str, "true") == 0 || strcasecmp(str, "on") == 0 || strcasecmp(str, "yes") == 0 || atoi(str) != 0)
    return 1;
  else
    return 0;
}
